isc.defineClass("SavedSearch", VStack);

isc.SavedSearch.addProperties({
    membersMargin: 10,
    dataSource: null,
    savedSearchDataSource: null,
    _editSavedSearchWindow: null,
    initWidget: function () {
        this.Super("initWidget", arguments);
        this.addMembers(this._createMembers());
        this._nameForm.getField("name").fetchData();
        this._listGrid.fetchData();   
    },
    _createMembers: function () {
        var _this = this;

        return [
            isc.HStack.create({
                membersMargin: 10,
                height: 25,
                width: 100,
                autoDraw: false,
                members:[
                    this._nameForm = isc.DynamicForm.create({
                        width: 200,
                        autoDraw: false,
                        fields: [
                            {   
                                name: "name",
                                editorType: "ComboBoxItem",
                                displayField: "name",
                                valueField: "id",
                                title: "Filter",
                                emptyDisplayValue: "All Records",
                                showAllOptions: true,
                                optionDataSource: this.savedSearchDataSource,
                                fetchMissingValues: false,
                                cachePickListResults: false,
                                // filter data by item datasource
                                optionCriteria: { 
                                    fieldName:"type", 
                                    operator:"equals",
                                    value: this.dataSource.ID
                                },
                                changed: function (form, item, value) {
                                    _this._onChangedFilter(item);
                                }
                            }
                        ]
                    }),
                    this._addSavedSearch = isc.Button.create({
                        title: "New search",
                        autoDraw: false,
                        width: 120,
                        click: function () {
                            _this._showAddSavedSearchWindow() 
                        }
                    }),
                    this._updateSavedSearch = isc.Button.create({
                        title: "Update search",
                        autoDraw: false,
                        width: 120,
                        disabled: true,
                        click: function () {
                            _this._showUpdateSavedSearchWindow() 
                        }
                    }),
                    this._removeSavedSearch = isc.Button.create({
                        title: "Remove search",
                        autoDraw: false,
                        width: 120,
                        disabled: true,
                        click: function () {
                            _this._onRemoveSavedSearch() 
                        }
                    })
                ]
            }),
            this._listGrid = isc.ListGrid.create({
                dataSource: this.dataSource,
                autoDraw: false,
                height: 450,
            })
        ];
    },
    _createEditSavedSearchWindow: function () {
        var _this = this;
        return isc.EditSavedSearchWindow.create({
            dataSource: this.savedSearchDataSource,
            itemDataSource: this.dataSource,
            autoDraw: false,
            updateSavedSearchCallback: function (record) {
                _this._onUpdateSavedSearch(record);
            }
        });
    },
    _showAddSavedSearchWindow: function () {
        if (this._editSavedSearchWindow == null) {
            this._editSavedSearchWindow = this._createEditSavedSearchWindow();
        }

        this._editSavedSearchWindow.record = null;
        this._editSavedSearchWindow.show();
    },
    _showUpdateSavedSearchWindow: function () {
        var record = this._nameForm.getField("name").getSelectedRecord();
        if (record == null) {
            return;
        }

        if (this._editSavedSearchWindow == null) {
            this._editSavedSearchWindow = this._createEditSavedSearchWindow();
        }

        this._editSavedSearchWindow.record = record;
        this._editSavedSearchWindow.show();
    },
    _onRemoveSavedSearch: function () {
        var nameComboBoxItem = this._nameForm.getField("name");
        var record = nameComboBoxItem.getSelectedRecord();
        if (record == null) {
            return;
        }

        nameComboBoxItem.getOptionDataSource().removeData(record);
        nameComboBoxItem.clearValue();
        this._updateSavedSearch.disable();
        this._removeSavedSearch.disable();
        this._listGrid.fetchData();
    },
    _onChangedFilter: function (item) {
        var record = item.getSelectedRecord();

        if (record == null) {
            this._updateSavedSearch.disable();
            this._removeSavedSearch.disable();
        } else {
            this._updateSavedSearch.enable();
            this._removeSavedSearch.enable();
        }

        if (record != null) {
            var newCriteria = isc.JSON.decode(record.criteria);
            this._listGrid.fetchData(newCriteria);
        } else {
            this._listGrid.fetchData();
        }
    },
    _onUpdateSavedSearch: function (record) {
        this._nameForm.getField("name").setValue(record.id);
        this._updateSavedSearch.enable();
        this._removeSavedSearch.enable();
        var criteria = isc.JSON.decode(record.criteria);
        this._listGrid.fetchData(criteria);
    }
});